/*
 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== spi_expander_test.c ========
 */

/* For usleep() */
#include <unistd.h>
#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/SPI.h>
#include <ti/drivers/UART.h>

/* Driver configuration */
#include "ti_drivers_config.h"

/* Static Variables */
static UART_Handle uart;
static int_fast32_t terminalWrite(const char* message);

/*
 *  ======== mainThread ========
 */
void* mainThread(void *arg0)
{
    SPI_Params spiParams;
    SPI_Handle masterSpi;
    SPI_Transaction transaction;
    UART_Params uartParams;
    char selectChar;
    char writeByteString[7] = "0x";
    uint16_t writeByte;
    uint16_t writeByteReversed = 0;
    uint16_t readByte;
    char readByteString[7] = "0x";
    bool retCode = false;

    /* Call driver init functions */
    GPIO_init();
    SPI_init();
    UART_init();

    /* Setting the NULL terminator of the write string */
    writeByteString[6] = 0;

    /* Create a UART with data processing off. */
    UART_Params_init (&uartParams);
    uartParams.writeDataMode = UART_DATA_BINARY;
    uartParams.readDataMode = UART_DATA_BINARY;
    uartParams.readReturnMode = UART_RETURN_FULL;
    uartParams.baudRate = 115200;

    uart = UART_open(TERMINAL_IO, &uartParams);

    if (uart == NULL)
    {
        while(1);
    }

    /* Open SPI as master (default) */
    SPI_Params_init (&spiParams);
    spiParams.frameFormat = SPI_POL0_PHA0;
    spiParams.bitRate = 4000000;
    masterSpi = SPI_open(SPI_EXPANDER, &spiParams);

    if (masterSpi == NULL)
    {
        while(1);
    }

    terminalWrite(
            "\r\n ----------- TPS272C45 SPI Expander Tester ----------- \r\n");

    do
    {
        terminalWrite("1) Write a byte \r\n");
        terminalWrite("2) Read status \r\n");
        terminalWrite("Choose an option: ");
        UART_read(uart, &selectChar, 1);

        if(selectChar == '1')
        {
            /* Reading in the byte from the user */
            terminalWrite("\r\nTell me what to write in hex (XXXX): ");
            UART_read(uart, writeByteString + 2, 4);
            writeByte = (uint16_t)strtol(writeByteString, NULL, 0);
            writeByteReversed = (writeByte & 0x0F00) << 4;
            writeByteReversed |= (writeByte & 0xF000) >> 4;
            writeByteReversed |= (writeByte & 0x00F0) >> 4;
            writeByteReversed |= (writeByte & 0x000F) << 4;

            /* Writing the SPI byte */
            transaction.count = 2;
            transaction.txBuf = (void *) &writeByteReversed;
            transaction.rxBuf = NULL;

            /* Perform SPI transfer */
            GPIO_write(SPI_EXPANDER_CS, 0);
            retCode = SPI_transfer(masterSpi, &transaction);
            GPIO_write(SPI_EXPANDER_CS, 1);

            if (retCode == true)
            {
                terminalWrite("\r\nSuccessfully wrote value of ");
                sprintf(readByteString, "0x%X", writeByte);
                terminalWrite(readByteString);
                terminalWrite("\r\n");
            }
            else
            {
                terminalWrite("\r\Error writing value of ");
                terminalWrite(writeByteString);
                terminalWrite("\r\n");
            }

        }
        else if (selectChar == '2')
        {
            /* Reading/Writing the SPI registers */
            transaction.count = 2;
            transaction.txBuf = &writeByteReversed;
            transaction.rxBuf = &readByte;

            GPIO_write(SPI_EXPANDER_CS, 0);
            retCode = SPI_transfer(masterSpi, &transaction);
            GPIO_write(SPI_EXPANDER_CS, 1);

            if(retCode == true)
            {
                terminalWrite("\r\nSuccessfully read value of ");
                sprintf(readByteString, "0x%X", (readByte & 0xFF));
                terminalWrite(readByteString);
                terminalWrite("\r\n");
            }
            else
            {
                terminalWrite("\r\nError reading byte value!!!\r\n");
            }

        }
        else
        {
            terminalWrite("\r\n**** Invalid Selection, Try Again ****\r\n");
        }

    } while (1);
}

static int_fast32_t terminalWrite(const char* message)
{
    return (UART_write(uart, message, strlen(message)));
}
